@@ -4,7 +4,7 @@ use v5.14;
 use warnings;
 use utf8;
 
-our $VERSION = "2.3301";
+our $VERSION = "2.3302";
 
 use Data::Dumper;
 {
@@ -17,7 +17,6 @@ use Carp;
 use List::Util qw(pairmap pairgrep);
 use Scalar::Util qw(looks_like_number);
 use Text::VisualWidth::PP 'vwidth';
-sub pwidth { vwidth $_[0] =~ s/\X\cH{1,2}//gr }
 
 ######################################################################
 use Exporter 'import';
@@ -72,6 +71,8 @@ our $osc_re          = qr{
 use constant SGR_RESET  => "\e[m";
 use constant OSC8_RESET => "\e]8;;\e\\";
 
+sub pwidth { vwidth $_[0] =~ s/\X\cH{1,2}//gr =~ s/${osc_re}//gr }
+
 sub IsPrintableLatin {
     return <<"END";
 +utf8::ASCII
@@ -464,7 +465,8 @@ sub fold {
 	my $l = pwidth($w1);
 	## prefix length
 	my $p = $opt->{prefix} eq '' ? 0 : vwidth($opt->{prefix});
-	if ($room + $l < $width - $p and $l + pwidth($w2) <= $width - $p) {
+	if ($room + $l < $width - $p and $l + pwidth($w2) <= $width - $p
+	    and pwidth($lead) > 0) {
 	    $folded = $lead;
 	    $_ = $w1 . pop_reset() . $_;
 	    $room += $l;
@@ -480,7 +482,7 @@ sub fold {
 			    (?: ($prohibition_re{end}) (?: \cH{1,2} \g{-1})* )+
 			  ) \z
 			}xp
-	    and ${^PREMATCH} ne ''
+	    and ${^PREMATCH} ne '' and pwidth(${^PREMATCH}) > 0
 	    and (my $w = pwidth $+{runout}) <= $opt->{runout}) {
 
 	    $folded = ${^PREMATCH};
@@ -583,8 +585,10 @@ sub retrieve {
     my $obj = shift;
     local *_ = \$obj->{text};
     return '' if not defined $_;
-    (my $folded, $_) = $obj->fold($_, @_);
-    $_ = undef if length == 0;
+    (my $folded, my $rest) = $obj->fold($_, @_);
+    die "panic: retrieve: no progress in fold"
+	if length $rest and length $rest >= length;
+    $_ = length $rest ? $rest : undef;
     $folded;
 }
 
@@ -629,7 +633,7 @@ Text::ANSI::Fold - Text folding library supporting ANSI terminal sequence and As
 
 =head1 VERSION
 
-Version 2.3301
+Version 2.3302
 
 =head1 SYNOPSIS
 
